﻿#include <windows.h>
#include <iostream>
#include <cstdint>

// Module: Module1

// Class Address (NotInheritable)
class Address {
public:
    static const uintptr_t EHL_Address = 0xC00F8000UL; // EHL_Address as ULong
    // EHL(AS-1560) Dout A0-A7
    //
    static const uintptr_t EHL_DoutA0 = 0xFD6B08B0ULL;
    static const uintptr_t EHL_DoutA1 = 0xFD6B08C0ULL;
    static const uintptr_t EHL_DoutA2 = 0xFD6B08D0ULL;
    static const uintptr_t EHL_DoutA3 = 0xFD6B08E0ULL;
    static const uintptr_t EHL_DoutA4 = 0xFD6B08F0ULL;
    static const uintptr_t EHL_DoutA5 = 0xFD6B0900ULL;
    static const uintptr_t EHL_DoutA6 = 0xFD6B0910ULL;
    static const uintptr_t EHL_DoutA7 = 0xFD6B0920ULL;
    //
    // EHL(AS-1560) Din A0-A7
    //
    static const uintptr_t EHL_DinA0 = 0xFD6B0830ULL;
    static const uintptr_t EHL_DinA1 = 0xFD6B0840ULL;
    static const uintptr_t EHL_DinA2 = 0xFD6B0850ULL;
    static const uintptr_t EHL_DinA3 = 0xFD6B0860ULL;
    static const uintptr_t EHL_DinA4 = 0xFD6B0870ULL;
    static const uintptr_t EHL_DinA5 = 0xFD6B0880ULL;
    static const uintptr_t EHL_DinA6 = 0xFD6B0890ULL;
    static const uintptr_t EHL_DinA7 = 0xFD6B08A0ULL;
};

// Declaration of kernel32.dll functions
// LoadLibrary function declaration
// Note: In C++ we directly use the Windows API functions from <windows.h>

// GetProcAddress function declaration
// (Also provided by <windows.h>)

// FreeLibrary function declaration
// (Also provided by <windows.h>)

// Declaration of custom DLL functions using function pointer types with __stdcall calling convention
typedef bool(__stdcall* InitializeWinIoType)();
typedef bool(__stdcall* GetPhysLongType)(void* PhysAddr, unsigned int& pPhysVal);
typedef bool(__stdcall* SetPhysLongType)(void* PhysAddr, unsigned int PhysVal);
typedef bool(__stdcall* ShutdownWinIoType)();

// Global variable hMod
HMODULE hMod = NULL;

// Function prototypes
void High_all_EHL_DoutA();
void Low_all_EHL_DoutA();
unsigned short Check_EHL_DinA();
void Read_EHL_DinA();
void btnQuit_Click();

int main()
{
    // Check if this is a 32 bit or 64 bit system
    if (sizeof(void*) == 4)
    {
        hMod = LoadLibraryA("WinIo32.dll");
    }
    else if (sizeof(void*) == 8)
    {
        hMod = LoadLibraryA("WinIo64.dll");
    }
    if (hMod == NULL)
    {
        std::cout << "Can't find WinIo dll." << std::endl;
    }

    void* pFunc = (void*)GetProcAddress(hMod, "InitializeWinIo");

    if (pFunc != NULL)
    {
        InitializeWinIoType InitializeWinIo = reinterpret_cast<InitializeWinIoType>(pFunc);

        // Init  DIO Driver
        bool Result = InitializeWinIo();

        if (!Result)
        {
            std::cout << "Error returned from Initin the same directory as your executable file" << std::endl;
            FreeLibrary(hMod);
        }
        std::cout << "InitializeWinIo" << std::endl;
    }
    // Check AS-1560G (EHL PCH Device)
    void* pFuncGet = (void*)GetProcAddress(hMod, "GetPhysLong");
    if (pFuncGet != NULL)
    {
        unsigned int EHL_DID_VID;

        GetPhysLongType GetPhysLong = reinterpret_cast<GetPhysLongType>(pFuncGet);
        GetPhysLong((void*)Address::EHL_Address, EHL_DID_VID);

        // Vender ID = 8086 ,  EHL  PCH Device ID = 4B00 (AS-3610)
        //
        if (EHL_DID_VID == 0x4B008086U)   // AS-1560G
        {
            std::cout << "AS-1560G DIO Sample for C++" << std::endl;
        }
        else
        {
            std::cout << "Not EHL Platform!!" << std::endl;
        }
    }

    std::cout << "High all Test" << std::endl;

    // Write High to all bits of DoutA
    High_all_EHL_DoutA();

    // Read all bits of DoutA
    Read_EHL_DinA();

    std::cout << "Low all Test" << std::endl;

    // Write Low to all bits of DoutA
    Low_all_EHL_DoutA();

    // Read all bits of DoutA
    Read_EHL_DinA();

    std::cout << "ShutDownWinIo" << std::endl;

    // ShutdownWinIo
    btnQuit_Click();

    return 0;
}

//
// Function High all EHL Dout A
//
void High_all_EHL_DoutA()
{
    void* pFuncSet = (void*)GetProcAddress(hMod, "SetPhysLong");
    void* pFuncGet = (void*)GetProcAddress(hMod, "GetPhysLong");

    if ((pFuncSet != NULL) && (pFuncGet != NULL))
    {
        unsigned int DoutA0, DoutA1, DoutA2, DoutA3, DoutA4, DoutA5, DoutA6, DoutA7;

        GetPhysLongType GetPhysLong = reinterpret_cast<GetPhysLongType>(pFuncGet);
        SetPhysLongType SetPhysLong = reinterpret_cast<SetPhysLongType>(pFuncSet);

        // High Dout A0
        GetPhysLong((void*)Address::EHL_DoutA0, DoutA0);
        DoutA0 = DoutA0 | 0x1U;
        SetPhysLong((void*)Address::EHL_DoutA0, DoutA0);

        // High Dout A1
        GetPhysLong((void*)Address::EHL_DoutA1, DoutA1);
        DoutA1 = DoutA1 | 0x1U;
        SetPhysLong((void*)Address::EHL_DoutA1, DoutA1);

        // High Dout A2
        GetPhysLong((void*)Address::EHL_DoutA2, DoutA2);
        DoutA2 = DoutA2 | 0x1U;
        SetPhysLong((void*)Address::EHL_DoutA2, DoutA2);

        // High Dout A3
        GetPhysLong((void*)Address::EHL_DoutA3, DoutA3);
        DoutA3 = DoutA3 | 0x1U;
        SetPhysLong((void*)Address::EHL_DoutA3, DoutA3);

        // High Dout A4
        GetPhysLong((void*)Address::EHL_DoutA4, DoutA4);
        DoutA4 = DoutA4 | 0x1U;
        SetPhysLong((void*)Address::EHL_DoutA4, DoutA4);

        // High Dout A5
        GetPhysLong((void*)Address::EHL_DoutA5, DoutA5);
        DoutA5 = DoutA5 | 0x1U;
        SetPhysLong((void*)Address::EHL_DoutA5, DoutA5);

        // High Dout A6
        GetPhysLong((void*)Address::EHL_DoutA6, DoutA6);
        DoutA6 = DoutA6 | 0x1U;
        SetPhysLong((void*)Address::EHL_DoutA6, DoutA6);

        // High Dout A7
        GetPhysLong((void*)Address::EHL_DoutA7, DoutA7);
        DoutA7 = DoutA7 | 0x1U;
        SetPhysLong((void*)Address::EHL_DoutA7, DoutA7);
    }
}

//
// Function Low all EHL Dout A
//
void Low_all_EHL_DoutA()
{
    void* pFuncSet = (void*)GetProcAddress(hMod, "SetPhysLong");
    void* pFuncGet = (void*)GetProcAddress(hMod, "GetPhysLong");

    if ((pFuncSet != NULL) && (pFuncGet != NULL))
    {
        unsigned int DoutA0, DoutA1, DoutA2, DoutA3, DoutA4, DoutA5, DoutA6, DoutA7;

        GetPhysLongType GetPhysLong = reinterpret_cast<GetPhysLongType>(pFuncGet);
        SetPhysLongType SetPhysLong = reinterpret_cast<SetPhysLongType>(pFuncSet);

        // Low Dout A0
        GetPhysLong((void*)Address::EHL_DoutA0, DoutA0);
        DoutA0 = DoutA0 & 0xFFFFFFFEU;
        SetPhysLong((void*)Address::EHL_DoutA0, DoutA0);

        // Low Dout A1
        GetPhysLong((void*)Address::EHL_DoutA1, DoutA1);
        DoutA1 = DoutA1 & 0xFFFFFFFEU;
        SetPhysLong((void*)Address::EHL_DoutA1, DoutA1);

        // Low Dout A2
        GetPhysLong((void*)Address::EHL_DoutA2, DoutA2);
        DoutA2 = DoutA2 & 0xFFFFFFFEU;
        SetPhysLong((void*)Address::EHL_DoutA2, DoutA2);

        // Low Dout A3
        GetPhysLong((void*)Address::EHL_DoutA3, DoutA3);
        DoutA3 = DoutA3 & 0xFFFFFFFEU;
        SetPhysLong((void*)Address::EHL_DoutA3, DoutA3);

        // Low Dout A4
        GetPhysLong((void*)Address::EHL_DoutA4, DoutA4);
        DoutA4 = DoutA4 & 0xFFFFFFFEU;
        SetPhysLong((void*)Address::EHL_DoutA4, DoutA4);

        // Low Dout A5
        GetPhysLong((void*)Address::EHL_DoutA5, DoutA5);
        DoutA5 = DoutA5 & 0xFFFFFFFEU;
        SetPhysLong((void*)Address::EHL_DoutA5, DoutA5);

        // Low Dout A6
        GetPhysLong((void*)Address::EHL_DoutA6, DoutA6);
        DoutA6 = DoutA6 & 0xFFFFFFFEU;
        SetPhysLong((void*)Address::EHL_DoutA6, DoutA6);

        // Low Dout A7
        GetPhysLong((void*)Address::EHL_DoutA7, DoutA7);
        DoutA7 = DoutA7 & 0xFFFFFFFEU;
        SetPhysLong((void*)Address::EHL_DoutA7, DoutA7);
    }
}

//
// Check all EHL DinA state.
//
unsigned short Check_EHL_DinA()
{
    void* pFuncSet = (void*)GetProcAddress(hMod, "SetPhysLong");
    void* pFuncGet = (void*)GetProcAddress(hMod, "GetPhysLong");

    // if ((pFuncSet != NULL) && (pFuncGet != NULL))
    //{
    unsigned int DinA0, DinA1, DinA2, DinA3, DinA4, DinA5, DinA6, DinA7;
    unsigned int DinA0_Rx, DinA1_Rx, DinA2_Rx, DinA3_Rx, DinA4_Rx, DinA5_Rx, DinA6_Rx, DinA7_Rx;
    unsigned short Check_result;

    Check_result = 0x0;

    GetPhysLongType GetPhysLong = reinterpret_cast<GetPhysLongType>(pFuncGet);
    SetPhysLongType SetPhysLong = reinterpret_cast<SetPhysLongType>(pFuncSet);

    // Delay 1 ms
    Sleep(1);

    // Get Din A0 Rx State.
    GetPhysLong((void*)Address::EHL_DinA0, DinA0);
    DinA0 = DinA0 & 0x2U;
    DinA0_Rx = DinA0;

    if (DinA0_Rx == 0x2U)
    {
        Check_result = Check_result | 0x1;
    }
    else
    {
        Check_result = Check_result & 0xFFFE;
    }

    // Get Din A1 Rx State.
    GetPhysLong((void*)Address::EHL_DinA1, DinA1);
    DinA1 = DinA1 & 0x2U;
    DinA1_Rx = DinA1;

    if (DinA1_Rx == 0x2U)
    {
        Check_result = Check_result | 0x2;
    }
    else
    {
        Check_result = Check_result & 0xFFFD;
    }

    // Get Din A2 Rx State.
    GetPhysLong((void*)Address::EHL_DinA2, DinA2);
    DinA2 = DinA2 & 0x2U;
    DinA2_Rx = DinA2;

    if (DinA2_Rx == 0x2U)
    {
        Check_result = Check_result | 0x4;
    }
    else
    {
        Check_result = Check_result & 0xFFFB;
    }

    // Get Din A3 Rx State.
    GetPhysLong((void*)Address::EHL_DinA3, DinA3);
    DinA3 = DinA3 & 0x2U;
    DinA3_Rx = DinA3;

    if (DinA3_Rx == 0x2U)
    {
        Check_result = Check_result | 0x8;
    }
    else
    {
        Check_result = Check_result & 0xFFF7;
    }

    // Get Din A4 Rx State.
    GetPhysLong((void*)Address::EHL_DinA4, DinA4);
    DinA4 = DinA4 & 0x2U;
    DinA4_Rx = DinA4;

    if (DinA4_Rx == 0x2U)
    {
        Check_result = Check_result | 0x10;
    }
    else
    {
        Check_result = Check_result & 0xFFEF;
    }

    // Get Din A5 Rx State.
    GetPhysLong((void*)Address::EHL_DinA5, DinA5);
    DinA5 = DinA5 & 0x2U;
    DinA5_Rx = DinA5;

    if (DinA5_Rx == 0x2U)
    {
        Check_result = Check_result | 0x20;
    }
    else
    {
        Check_result = Check_result & 0xFFDF;
    }

    // Get Din A6 Rx State.
    GetPhysLong((void*)Address::EHL_DinA6, DinA6);
    DinA6 = DinA6 & 0x2U;
    DinA6_Rx = DinA6;

    if (DinA6_Rx == 0x2U)
    {
        Check_result = Check_result | 0x40;
    }
    else
    {
        Check_result = Check_result & 0xFFBF;
    }

    // Get Din A7 Rx State.
    GetPhysLong((void*)Address::EHL_DinA7, DinA7);
    DinA7 = DinA7 & 0x2U;
    DinA7_Rx = DinA7;

    if (DinA7_Rx == 0x2U)
    {
        Check_result = Check_result | 0x80;
    }
    else
    {
        Check_result = Check_result & 0xFF7F;
    }

    return Check_result;
    //}
}

//
// Function btnQuit_Click
//
void btnQuit_Click()
{
    void* pFunc = (void*)GetProcAddress(hMod, "ShutdownWinIo");

    if (pFunc != NULL)
    {
        ShutdownWinIoType ShutdownWinIo = reinterpret_cast<ShutdownWinIoType>(pFunc);

        ShutdownWinIo();
        FreeLibrary(hMod);
    }
}

//
// Function Read EHL DinA
//
void Read_EHL_DinA()
{
    void* pFunc = (void*)GetProcAddress(hMod, "GetPhysLong");

    if (pFunc != NULL)
    {
        unsigned int PhysVal0, PhysVal1, PhysVal2, PhysVal3,
            PhysVal4, PhysVal5, PhysVal6, PhysVal7;
        unsigned short Data_DinA;

        Data_DinA = 0x0;

        GetPhysLongType GetPhysLong = reinterpret_cast<GetPhysLongType>(pFunc);

        // Call WinIo to get value

        GetPhysLong((void*)Address::EHL_DinA0, PhysVal0);
        GetPhysLong((void*)Address::EHL_DinA1, PhysVal1);
        GetPhysLong((void*)Address::EHL_DinA2, PhysVal2);
        GetPhysLong((void*)Address::EHL_DinA3, PhysVal3);
        GetPhysLong((void*)Address::EHL_DinA4, PhysVal4);
        GetPhysLong((void*)Address::EHL_DinA5, PhysVal5);
        GetPhysLong((void*)Address::EHL_DinA6, PhysVal6);
        GetPhysLong((void*)Address::EHL_DinA7, PhysVal7);

        PhysVal0 = PhysVal0 & 0x2U;
        PhysVal1 = PhysVal1 & 0x2U;
        PhysVal2 = PhysVal2 & 0x2U;
        PhysVal3 = PhysVal3 & 0x2U;
        PhysVal4 = PhysVal4 & 0x2U;
        PhysVal5 = PhysVal5 & 0x2U;
        PhysVal6 = PhysVal6 & 0x2U;
        PhysVal7 = PhysVal7 & 0x2U;

        if (PhysVal0 == 0x2U)
        {
            Data_DinA = Data_DinA | 0x1;
            std::cout << "Din_A0 - High" << std::endl;
        }
        else
        {
            Data_DinA = Data_DinA & 0xFFFE;
            std::cout << "Din_A0 - Low" << std::endl;
        }

        if (PhysVal1 == 0x2U)
        {
            Data_DinA = Data_DinA | 0x2;
            std::cout << "Din_A1 - High" << std::endl;
        }
        else
        {
            Data_DinA = Data_DinA & 0xFFFD;
            std::cout << "Din_A1 - Low" << std::endl;
        }

        if (PhysVal2 == 0x2U)
        {
            Data_DinA = Data_DinA | 0x4;
            std::cout << "Din_A2 - High" << std::endl;
        }
        else
        {
            Data_DinA = Data_DinA & 0xFFFB;
            std::cout << "Din_A2 - Low" << std::endl;
        }

        if (PhysVal3 == 0x2U)
        {
            Data_DinA = Data_DinA | 0x8;
            std::cout << "Din_A3 - High" << std::endl;
        }
        else
        {
            Data_DinA = Data_DinA & 0xFFF7;
            std::cout << "Din_A3 - Low" << std::endl;
        }

        if (PhysVal4 == 0x2U)
        {
            Data_DinA = Data_DinA | 0x10;
            std::cout << "Din_A4 - High" << std::endl;
        }
        else
        {
            Data_DinA = Data_DinA & 0xFFEF;
            std::cout << "Din_A4 - Low" << std::endl;
        }

        if (PhysVal5 == 0x2U)
        {
            Data_DinA = Data_DinA | 0x20;
            std::cout << "Din_A5 - High" << std::endl;
        }
        else
        {
            Data_DinA = Data_DinA & 0xFFDF;
            std::cout << "Din_A5 - Low" << std::endl;
        }

        if (PhysVal6 == 0x2U)
        {
            Data_DinA = Data_DinA | 0x40;
            std::cout << "Din_A6 - High" << std::endl;
        }
        else
        {
            Data_DinA = Data_DinA & 0xFFBF;
            std::cout << "Din_A6 - Low" << std::endl;
        }

        if (PhysVal7 == 0x2U)
        {
            Data_DinA = Data_DinA | 0x80;
            std::cout << "Din_A7 - High" << std::endl;
        }
        else
        {
            Data_DinA = Data_DinA & 0xFF7F;
            std::cout << "Din_A7 - Low" << std::endl;
        }
    }
}

// End of Module1

